@@ -68,9 +68,14 @@ class HostConnectionCache(dict):
         # Return the value either way
         return dict.__getitem__(self, real_key)
 
-    def __delitem__(self, key):
-        return dict.__delitem__(self, join_host_strings(*normalize(key)))
+    def __setitem__(self, key, value):
+        return dict.__setitem__(self, normalize_to_string(key), value)
 
+    def __delitem__(self, key):
+        return dict.__delitem__(self, normalize_to_string(key))
+
+    def __contains__(self, key):
+        return dict.__contains__(self, normalize_to_string(key))
 
 def normalize(host_string, omit_port=False):
     """
@@ -127,6 +132,13 @@ def join_host_strings(user, host, port=None):
     return "%s@%s%s" % (user, host, port_string)
 
 
+def normalize_to_string(host_string):
+    """
+    normalize() returns a tuple; this returns another valid host string.
+    """
+    return join_host_strings(*normalize(host_string))
+
+
 def connect(user, host, port):
     """
     Create and return a new SSHClient instance connected to given host.
@@ -272,7 +284,6 @@ def prompt_for_password(prompt=None, no_colon=False, stream=None):
     # returning the empty string, and to avoid unnecessary network overhead.)
     while not new_password:
         print("Sorry, you can't enter an empty password. Please try again.")
-        password_prompt = base_password_prompt + ": "
         new_password = getpass.getpass(password_prompt, stream)
     return new_password
 
